/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include "ns3/gpsr-module.h"
#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/internet-module.h"
#include "ns3/mobility-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/wifi-module.h"
#include "ns3/v4ping-helper.h"
#include "ns3/udp-server.h"
#include "ns3/udp-client.h"
#include "ns3/udp-client-server-helper.h"
#include "ns3/flow-monitor-helper.h"
#include "src/stats/model/gnuplot.h"
#include "ns3/netanim-module.h"

#include <iostream>
#include <algorithm>
#include <string>
#include <cmath>

using namespace ns3;

int numNodes = 0;
double duration = 60.0;
double MapRX = 830;
double MapLX = 100;
double MapUY = 100;
double MapDY = 500;


std::string ns2mobfile = "VanetsJournal/BonMotion/scenario.ns_movements";

void
CreateNode (double posX, double posY, double dstX, double dstY, double vel)
{
    std::ofstream os;
    os.open ((ns2mobfile).c_str(), std::ios::app);
    
    os << "$node_("<< numNodes << ") set X_ " << posX << "\n"
       << "$node_("<< numNodes << ") set Y_ " << posY << "\n"
       << "$node_("<< numNodes << ") set Z_ 0\n"
       << "$ns_ at 2.5 \"$node_("<< numNodes << ") setdest " 
       << dstX << " "<< dstY << " " << vel << "\"\n";
    
    if (vel!= 0)
      os << "$ns_ at " << CalculateDistance (Vector(posX, posY, 0), Vector(dstX, dstY, 0)) / (vel) << " \"$node_("<< numNodes << ") setdest " 
         << posX << " " << posY << " " << vel << "\"\n\n";
    
    
    
    numNodes ++;

}

void
CreateRoad (uint32_t vehNum, double StposX, double StposY, double distance, double vel, char direction)
{
    double dest;
    double dist = 0.0;
    
    switch (direction)
    {
      case 'U':
        dest = 10;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateNode (StposX, StposY - dist, StposX, StposY - dest, vel);
          dist += distance;
        }
        break;
      case 'D':
        dest = 760;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateNode (StposX, StposY + dist, StposX, StposY + dest, vel);
          dist += distance;
        }
        break;
      case 'L':
        dest = 10;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateNode (StposX - dist, StposY, StposX - dest, StposY, vel);
          dist += distance;
        }
        break;
      case 'R':
        dest = 760;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateNode (StposX + dist, StposY, StposX + dest, StposY, vel);
          dist += distance;
        }
        break;
      default:
        NS_FATAL_ERROR ("No such direction!");
    }    
}


void
CreateLoopNode (double posX, double posY, double dstX, double dstY, double vel, char direction)
{
    std::ofstream os;
    double t = 1.0;
    char d;
    os.open ((ns2mobfile).c_str(), std::ios::app);
    
    

      switch (direction)
      {
        case 'U':
              d = 'U';
              os << "$node_("<< numNodes << ") set X_ " << posX << "\n"
                 << "$node_("<< numNodes << ") set Y_ " << posY << "\n"
                 << "$node_("<< numNodes << ") set Z_ 0\n"
                 << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                 << dstX << " "<< MapUY << " " << vel << "\"\n";              
              
              if (vel!= 0)
                while (t<duration)
                {
                  if (d == 'U')
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(dstX, MapUY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << posX << " " << MapDY << " " << vel << "\"\n";
                    posY = MapUY;
                    d = 'D';
                  }
                  else
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(dstX, MapDY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << posX << " " << MapUY << " " << vel << "\"\n";
                    posY = MapDY;
                    d = 'U';
                  }
                };
          break;
        case 'D':
              d = 'D';
              os << "$node_("<< numNodes << ") set X_ " << posX << "\n"
                 << "$node_("<< numNodes << ") set Y_ " << posY << "\n"
                 << "$node_("<< numNodes << ") set Z_ 0\n"
                 << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                 << dstX << " "<< MapDY << " " << vel << "\"\n";              
              
              if (vel!= 0)
                while (t<duration)
                {
                  if (d == 'D')
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(dstX, MapDY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << posX << " " << MapUY << " " << vel << "\"\n";
                    posY = MapDY;
                    d = 'U';
                  }
                  else
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(dstX, MapUY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << posX << " " << MapDY << " " << vel << "\"\n";
                    posY = MapUY;
                    d = 'D';
                  }
                };
          break;
        case 'L':
              d = 'L';
              os << "$node_("<< numNodes << ") set X_ " << posX << "\n"
                 << "$node_("<< numNodes << ") set Y_ " << posY << "\n"
                 << "$node_("<< numNodes << ") set Z_ 0\n"
                 << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                 << MapLX << " "<< posY << " " << vel << "\"\n";              
              
              if (vel!= 0)
                while (t<duration)
                {
                  if (d == 'L')
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(MapLX, dstY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << MapRX << " " << posY << " " << vel << "\"\n";
                    posX = MapLX;
                    d = 'R';
                  }
                  else
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(MapRX, dstY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << MapLX << " " << posY << " " << vel << "\"\n";
                    posX = MapRX;
                    d = 'L';
                  }
                };
          break;
        case 'R':
          d = 'R';
              os << "$node_("<< numNodes << ") set X_ " << posX << "\n"
                 << "$node_("<< numNodes << ") set Y_ " << posY << "\n"
                 << "$node_("<< numNodes << ") set Z_ 0\n"
                 << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                 << MapRX << " "<< posY << " " << vel << "\"\n";              
              
              if (vel!= 0)
                while (t<duration)
                {
                  if (d == 'R')
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(MapRX, dstY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << MapLX << " " << posY << " " << vel << "\"\n";
                    posX = MapRX;
                    d = 'L';
                  }
                  else
                  {
                    t += CalculateDistance (Vector(posX, posY, 0), Vector(MapLX, dstY, 0)) / (vel);                    
                    os << "$ns_ at " << t << " \"$node_("<< numNodes << ") setdest " 
                           << MapRX << " " << posY << " " << vel << "\"\n";
                    posX = MapLX;
                    d = 'R';
                  }
                };
          break;
        default:
          NS_FATAL_ERROR ("No such direction!");
      }  
    numNodes ++;
}

void
CreateLoopRoad (uint32_t vehNum, double StposX, double StposY, double distance, double vel, char direction)
{
    double dest;
    double dist = 0.0;
    
    switch (direction)
    {
      case 'U':
        dest = 10;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateLoopNode (StposX, StposY - dist, StposX, StposY - dest, vel, direction);
          dist += distance;
        }
        break;
      case 'D':
        dest = 760;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateLoopNode (StposX, StposY + dist, StposX, StposY + dest, vel, direction);
          dist += distance;
        }
        break;
      case 'L':
        dest = 10;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateLoopNode (StposX - dist, StposY, StposX - dest, StposY, vel, direction);
          dist += distance;
        }
        break;
      case 'R':
        dest = 760;
        for (uint32_t i = 0; i < vehNum; ++i)
        {
          CreateLoopNode (StposX + dist, StposY, StposX + dest, StposY, vel, direction);
          dist += distance;
        }
        break;
      default:
        NS_FATAL_ERROR ("No such direction!");
    }    
}

int 
main (int argc, char *argv[])
{
  
  CommandLine cmd;  
  
  std::ofstream os;
  os.open ((ns2mobfile).c_str(), std::ios::trunc);
  os.close();
  
//  
//  CreateLoopRoad(13, 160, MapUY,15, 11.0, 'D');
//  CreateLoopRoad(12, 160, 150, 13, 11.0, 'U');
//  CreateLoopRoad(10, MapLX, 160, 20, 10.0, 'R');
//  CreateLoopRoad(15, MapLX, 310, 15, 5.0, 'R');
  
  
//  CreateLoopRoad(15, 160, MapUY,15, 2.0, 'D');
//  CreateLoopRoad(15, 310, 150, 13, 2.0, 'U');
//  CreateLoopRoad(15, 460, 200, 13, 2.0, 'D');
//  CreateLoopRoad(15, MapLX, 160, 15, 2.0, 'R');
//  CreateLoopRoad(15, MapLX, 310, 15, 2.0, 'R');
//  CreateLoopRoad(15, MapLX, 310, 15, 2.0, 'R');
//  CreateLoopRoad(10, MapLX, 310, 15, 2.0, 'R');
//  //  CreateRoad(15, 100.0, 310.0 , 8.0, 12.0, 'R');
  
  
//  CreateLoopRoad(15, 160, MapUY,15, 2.0, 'D');
//  CreateLoopRoad(15, 310, 150, 13, 2.0, 'U');
//  CreateLoopRoad(15, 460, 200, 13, 2.0, 'D');
//  CreateLoopRoad(15, MapLX, 160, 15, 2.0, 'R');
  CreateLoopRoad(50, MapLX, 310, 15, 2.0, 'R');
  CreateLoopRoad(50, MapRX, 310, 15, 2.0, 'L');
  
//  CreateLoopRoad(25, MapLX, 160, 15, 2.0, 'R');
//  CreateLoopRoad(25, MapRX, 460, 15, 2.0, 'L');
  
//  CreateLoopRoad(10, MapLX, 310, 15, 2.0, 'R');
  //  CreateRoad(15, 100.0, 310.0 , 8.0, 12.0, 'R');
}